MODULE zdfqiao
   !!======================================================================
   !!                       ***  MODULE  zdfqiao  ***
   !! Qiao module      : vertical mixing enhancement due to surface waves 
   !!======================================================================
   !! History :  3.6  !  2014-10  (E. Clementi)  Original code
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !!   qiao_init      
   !!   zdf_qiao        : compute Qiao parameters
   !!----------------------------------------------------------------------

   USE iom             ! I/O manager library
   USE in_out_manager  ! I/O manager
   USE lib_mpp         ! distribued memory computing library
   USE sbc_oce         ! Surface boundary condition: ocean fields
   USE zdf_oce
   USE sbcwave         ! wave module
   USE dom_oce
   
   !!----------------------------------------------------------------------
   !!   qiao_init       : compute QBv: Qiao terms to be added to vertical eddy
   !!                     diffusivity and viscosity coefficients 
   !!----------------------------------------------------------------------

   IMPLICIT NONE
   PRIVATE

   PUBLIC   zdf_qiao    ! routine called in zdf_ric

   REAL(wp),PUBLIC,ALLOCATABLE,DIMENSION (:,:,:)     :: QBv, QBvu, QBvv

   !!----------------------------------------------------------------------
   !! NEMO/OPA 4.0 , NEMO Consortium (2011) 
   !! $Id: $
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE zdf_qiao( kt )
      !!---------------------------------------------------------------------
      !!                     ***  ROUTINE zdf_qiao ***
      !!
      !! ** Purpose :Compute the Qiao term (QBv) to be added to
      !!             vertical viscosity and diffusivity coeffs.  
      !!
      !! ** Method  :QBv = alpha * A * Us(0) * exp (3 * k * z)
      !!             
      !! ** action  :Compute the Qiao wave dependent term 
      !!             only if ln_wave=.true.
      !!               
      !!---------------------------------------------------------------------
      INTEGER, INTENT( in  ) ::  kt   ! ocean time step
      INTEGER                ::  ierror   ! return error code
      INTEGER                ::  jj,ji,jk
      !!---------------------------------------------------------------------
      !!----------------------------------------------------------------------
      !
      !
      !                                         ! -------------------- !
      IF( kt == nit000 ) THEN                   ! First call kt=nit000 !
      !                                         ! -------------------- !
         ALLOCATE(QBv(jpi,jpj,jpk))
         ALLOCATE(QBvu(jpi,jpj,jpk))
         ALLOCATE(QBvv(jpi,jpj,jpk))

      ENDIF

         QBv (:,:,:) = 0.0
         QBvu(:,:,:) = 0.0
         QBvv(:,:,:) = 0.0
      !
      ! Compute the Qiao term Bv (QBv) to be added to
      ! vertical viscosity and diffusivity
      ! QBv = alpha * A * Us(0) * exp (3 * k * z)
      ! alpha here is set to 1
      !---------------------------------------------------------------------------------
      !
      IF ( ln_wave ) THEN
         DO jk = 1, jpk
          DO jj = 1, jpjm1
            DO ji = 1, jpim1

               QBv(ji,jj,jk) = 1.0 * 0.353553 * swh(ji,jj) * tsd2d(ji,jj) *     &
               &              exp(3.0 * wnum(ji,jj) *                            &                     
               &              (-MIN( gdept_0(ji  ,jj  ,jk) , gdept_0(ji+1,jj  ,jk),     & 
               &               gdept_0(ji  ,jj+1,jk),gdept_0(ji+1,jj+1,jk))))

            ENDDO
          ENDDO
         ENDDO

         QBv(jpi,:,:)=QBv(jpim1,:,:)
         QBv(:,jpj,:)=QBv(:,jpjm1,:)

         !
         ! Interpolate Qiao parameter QBv into the grid_U and grid_V
         !-------------------------------------------------

         DO jk = 1, jpk
           DO jj = 1, jpjm1
            DO ji = 1, jpim1

               QBvu(ji,jj,jk)=0.5 * ( 2. - umask(ji,jj,jk) ) *                 &
               &           ( QBv(ji,jj,jk) * tmask(ji,jj,jk)                   & 
               &           + QBv(ji+1,jj,jk) * tmask(ji+1,jj,jk) )               
               QBvv(ji,jj,jk)=0.5 * ( 2. - vmask(ji,jj,jk) ) *                 &
               &           ( QBv(ji,jj,jk) * tmask(ji,jj,jk)                   &
               &           + QBv(ji,jj+1,jk) * tmask(ji,jj+1,jk) )

             ENDDO
            ENDDO
          ENDDO
!
         QBvu(jpi,:,:)=QBvu(jpim1,:,:)
         QBvu(:,jpj,:)=QBvu(:,jpjm1,:)
         QBvv(jpi,:,:)=QBvv(jpim1,:,:)
         QBvv(:,jpj,:)=QBvv(:,jpjm1,:)
!
        ELSE
        !
        CALL ctl_stop( 'STOP', 'To use Qiao formulation you have to set: ln_wave=.true.')
        !
        ENDIF
!
   END SUBROUTINE zdf_qiao
      
   !!======================================================================
END MODULE zdfqiao
